define([    'shader!vertShader.vert',
            'shader!compShader.frag',
            'Abubu/Abubu'
            ],
function(   vertShader,
            compShader,
            Abubu
            ){
"use strict" ;

/*========================================================================
 * Global Parameters
 *========================================================================
 */
var log = console.log ;
var env = {} ;
window.env = env ;

/*========================================================================
 * Initialization of the GPU and Container
 *========================================================================
 */
function loadWebGL()
{
    env.width = 512 ;
    env.height = 512 ;

/*------------------------------------------------------------------------
 * Access and setup canvas 
 *------------------------------------------------------------------------
 */
    var canvas_1 = document.getElementById("canvas_1") ;
    canvas_1.width = env.width ;
    canvas_1.height = env.height ;
    
    var txtr = new Abubu.Float32Texture(env.width,env.height) ;
    
    env.r = 0.6 ;
    env.th = 0.1 ;

    /* Setup a solver */
    env.renderer = new Abubu.Solver( {
        fragmentShader  : compShader.value,
        vertexShader    : vertShader.value,
        uniforms : { 
            rr : { type : 'f', value : .6 } ,
            theta : { type : 'f', value : 0.1 } ,
        },
        renderTargets:{
            outcolor : { location : 0, target : txtr } 
        }
    } ) ;

    env.plt = new Abubu.Plot2D({
            target : txtr ,
            channel: 'r' ,
            minValue : 0 ,
            maxValue : 30 ,
            colorbar : true ,
            canvas  : canvas_1 ,
        } ) ;
    env.plt.init() ;

    createGui() ;

    run() ;
}/*  End of loadWebGL  */
function run(){
    env.renderer.render() ;
    env.plt.render() ;
}

function createGui(){
    var gui = new Abubu.Gui() ;
    var pnl = gui.addPanel() ;
    pnl.add(env, 'r').step(0.01).onChange(function(){
        env.renderer.uniforms.rr.value = env.r ;
        run() ;
    } ) ;
    pnl.add(env, 'th').step(0.01).onChange(function(){
        env.renderer.uniforms.theta.value = env.th ;
        run() ;
    } )  ;
}
loadWebGL() ;

} ) ;   // End of require 
